from pydantic import BaseModel

from .resource import Resource


class VideoCategorySnippet(BaseModel):
    title: str
    assignable: bool
    channel_id: str


class VideoCategory(Resource):
    id: str
    snippet: VideoCategorySnippet



from typing import Any, Iterator

from ...models import Comment, CommentThread
from ...schemas import (
    CommentThreadFilter,
    CommentThreadOptionalParameters,
    CommentThreadPart,
    YouTubeListResponse,
    YouTubeRequest,
    YouTubeResponse,
)
from ..resource import YouTubeResource


class YouTubeCommentThread(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)
        self.request_schema: YouTubeRequest = None
        self.next_page_token: str = None

    def parse_author(self, comment_snippet: dict) -> dict:
        author: dict = dict()
        author['display_name'] = comment_snippet['authorDisplayName']
        author['profile_image_url'] = comment_snippet['authorProfileImageUrl']
        author['channel_url'] = comment_snippet['authorChannelUrl']
        author['channel_id'] = comment_snippet['authorChannelId']['value']
        return author

    def parse_toplevel_comment(self, top_level_comment: dict) -> dict:
        comment: dict = dict()
        snippet: dict = dict()
        snippet['author'] = self.parse_author(top_level_comment['snippet'])
        snippet['channel_id'] = top_level_comment['snippet']['channelId']
        snippet['video_id'] = top_level_comment['snippet']['videoId']
        snippet['text_display'] = top_level_comment['snippet']['textDisplay']
        snippet['text_original'] = top_level_comment['snippet']['textOriginal']
        snippet['can_rate'] = top_level_comment['snippet']['canRate']
        snippet['viewer_rating'] = top_level_comment['snippet']['viewerRating']
        snippet['like_count'] = top_level_comment['snippet']['likeCount']
        if top_level_comment['snippet'].get('moderationStatus'):
            snippet['moderation_status'] = top_level_comment['snippet'][
                'moderationStatus'
            ]
        else:
            comment['moderation_status'] = ''
        snippet['published_at'] = top_level_comment['snippet']['publishedAt']
        snippet['updated_at'] = top_level_comment['snippet']['updatedAt']
        if top_level_comment['snippet'].get('parentId'):
            snippet['parent_id'] = top_level_comment['snippet']['parentId']
        else:
            snippet['parent_id'] = None
        comment['snippet'] = snippet
        comment['id'] = top_level_comment['id']
        return comment

    def parse_snippets(self, snippet_data: dict[str, Any]) -> dict[str, Any]:
        parsed_snippet: dict[str, Any] = dict()
        parsed_snippet['channel_id'] = snippet_data['channelId']
        parsed_snippet['video_id'] = snippet_data.get('videoId', '')
        parsed_snippet['can_reply'] = snippet_data.get('canReply', True)
        parsed_snippet['is_public'] = snippet_data.get('isPublic', True)
        parsed_snippet['total_reply_count'] = snippet_data['totalReplyCount']
        parsed_snippet['top_level_comment'] = self.parse_toplevel_comment(
            snippet_data['topLevelComment']
        )
        return dict(snippet=parsed_snippet)

    def parse_item(self, item: dict) -> CommentThread:
        id_data: dict = self.parse_id(item)
        snippet_data: dict = self.parse_snippets(item['snippet'])
        id_data.update(snippet_data)
        return CommentThread(**id_data, replies=[])

    def parse_youtube_threadlist_response(
        self, youtube_list_response: dict
    ) -> YouTubeListResponse:
        youtube_result: YouTubeListResponse = YouTubeListResponse(
            kind=youtube_list_response['kind'],
            etag=youtube_list_response['etag'],
            items=self.parse_items(youtube_list_response['items']),
        )
        return youtube_result

    def find_video_comments(self, request: YouTubeRequest) -> YouTubeListResponse:
        """Get a particular video's comments."""
        comment_thread_req = self.youtube_client.commentThreads().list(
            **self.create_request_dict(request)
        )
        comment_thread_resp = comment_thread_req.execute()
        return self.parse_youtube_list_response(comment_thread_resp)

    def find_all_channel_comments(self, request: YouTubeRequest) -> YouTubeListResponse:
        """Get a particular channels's comments."""
        request_dict: dict = self.create_request_dict(request)
        find_channel_comments: dict = self.youtube_client.commentThreads().list(
            **request_dict
        )
        find_channel_comments_resp: dict = find_channel_comments.execute()
        return self.parse_youtube_list_response(find_channel_comments_resp)

    # TODO create a custom parser
    def get_comment(self, comment_id: str) -> YouTubeListResponse:
        request_filter: CommentThreadFilter = CommentThreadFilter(id=[comment_id])
        part: CommentThreadPart = CommentThreadPart(part=['id', 'snippet'])
        optional_params: CommentThreadOptionalParameters = (
            CommentThreadOptionalParameters()
        )
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        find_comment_request: dict = self.youtube_client.comments().list(**request_dict)
        find_comment_result: dict = find_comment_request.execute()
        return find_comment_result

    # TODO create a custom parser
    def get_comments(self, comment_ids: list[str]) -> YouTubeListResponse:
        request_filter: CommentThreadFilter = CommentThreadFilter(id=comment_ids)
        part: CommentThreadPart = CommentThreadPart(part=['id', 'snippet'])
        optional_params: CommentThreadOptionalParameters = (
            CommentThreadOptionalParameters()
        )
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        find_comment_request: dict = self.youtube_client.comments().list(**request_dict)
        find_comment_result: dict = find_comment_request.execute()
        return find_comment_result

    # TODO create a custom parser
    def get_comment_replies(self, comment_id: str) -> YouTubeListResponse:
        comment_reply_request = self.youtube_client.comments().list(
            part='snippet,id', parentId=comment_id
        )
        comment_reply_response = comment_reply_request.execute()
        return comment_reply_response

    # TODO create a custom parser
    def insert_comment(self, video_id: str, comment: str) -> CommentThread:
        insert_comment_request = self.youtube_client.commentThreads().insert(
            part='snippet',
            body={
                'snippet': {
                    'videoId': video_id,
                    'topLevelComment': {'snippet': {'textOriginal': comment}},
                }
            },
        )
        insert_comment_response = insert_comment_request.execute()
        return insert_comment_response

    def reply_to_comment(self, comment_id: str, comment: str) -> Comment:
        comment_reply_request = self.youtube_client.comments().insert(
            part='snippet,id',
            body={'snippet': {'parentId': comment_id, 'textOriginal': comment}},
        )
        comment_reply_response = comment_reply_request.execute()
        return comment_reply_response

    def update_comment(self, comment_id: str, comment: str) -> Comment:
        update_comment_request = self.youtube_client.comments().update(
            part='snippet',
            body={'id': comment_id, 'snippet': {'textOriginal': comment}},
        )
        update_comment_response = update_comment_request.execute()
        return update_comment_response

    def delete_comment(self, comment_id: str) -> None:
        delete_comment_request = self.youtube_client.comments().delete(id=comment_id)
        delete_comment_request.execute()

    def __iter__(self):
        return self

    def __next__(self) -> list[Comment]:
        self.request_schema.optional_parameters.pageToken = self.next_page_token
        response: YouTubeResponse = self.find_video_comments(self.request_schema)
        self.next_page_token = response.nextPageToken
        if not self.next_page_token:
            raise StopIteration()
        return response.items

    def get_comments_iterator(self, request_schema: YouTubeResponse) -> Iterator:
        self.request_schema = request_schema
        return self



from datetime import datetime
from enum import Enum, auto

from pydantic import BaseModel, Field

from .resource import Resource
from .thumbnail import Thumbnail


class LiveBroadcastContent(Enum):
    Upcoming = auto()
    Live = auto()
    Null = auto()


class Search(Resource):
    resource_id: str
    resource_type: str
    published_at: datetime
    channel_id: str
    title: str
    description: str
    channel_title: str
    live_broadcast_content: str
    publish_time: str
    thumbnails: Thumbnail

    @property
    def url(self) -> str:
        return f'https://www.youtube.com/watch?v={self.resource_id}'



from typing import Optional

from pydantic import BaseModel


class VideoReportReasonSchema(BaseModel):
    videoId: str
    reasomId: str
    secondaryReasonId: Optional[str] = None
    comments: Optional[str] = None
    language: Optional[str] = None



from typing import Optional

from .youtube_list_response import YouTubeListResponse


class YouTubeResponse(YouTubeListResponse):
    nextPageToken: Optional[str] = ''
    prevPageToken: Optional[str] = ''



from enum import Enum, auto

from pydantic import BaseModel


class TimingType(Enum):
    OffsetFromStart = auto()
    OffsetFromEnd = auto()


class WaterMark(BaseModel):
    timing_type: TimingType
    offset_ms: int
    duration_ms: int
    position_type: str
    corner_position: str
    image_url: str
    image_bytes: bytes
    target_channel_id: str



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from ..models.resource import Resource
from .localized import Localized
from .thumbnail import Thumbnail


class Snippet(BaseModel):
    title: str
    channel_id: str
    channel_title: str
    description: str
    thumbnails: Thumbnail
    published_at: datetime
    default_language: str = None
    localized: Localized


class Status(BaseModel):
    privacy_status: str


class ContentDetails(BaseModel):
    item_count: int


class Playlist(Resource):
    id: str
    snippet: Optional[Snippet] = None
    status: Optional[Status] = None
    content_details: Optional[ContentDetails] = None



from .activity import ActivityFilter, ActivityOptionalParameters, ActivityPart
from .channel import ChannelFilter, ChannelOptionalParameters, ChannelPart
from .channel_section import (
    ChannelSectionFilter,
    ChannelSectionOptionalParameters,
    ChannelSectionPart,
    InsertChannelSection,
    InsertChannelSectionContentDetails,
    InsertChannelSectionSnippet,
)
from .comment_thread import CommentThreadFilter, CommentThreadOptionalParameters, CommentThreadPart
from .language_response import LanguageResponse
from .page_info import PageInfo
from .playlist import (
    CreatePlaylist,
    CreatePlaylistSnippet,
    CreateStatus,
    PlaylistFilter,
    PlaylistOptionalParameters,
    PlaylistPart,
)
from .playlist_item import (
    CreatePlaylistItem,
    CreatePlaylistItemSnippet,
    PlaylistItemFilter,
    PlaylistItemOptionalParameters,
    PlaylistItemPart,
    VideoResourceId,
)
from .rating import YouTubeRatingResponse
from .region_response import RegionResponse
from .report_abuse import VideoReportAbuse
from .search import SearchFilter, SearchOptionalParameters, SearchPart
from .subscription import SubscriptionFilter, SubscriptionOptionalParameters, SubscriptionPart
from .thumbnail_set_response import ThumbnailSetResponse
from .video import (
    UploadVideo,
    UploadVideoLocalizations,
    UploadVideoRecordingDetails,
    UploadVideoSnippet,
    UploadVideoStatus,
    VideoFilter,
    VideoOptionalParameters,
    VideoPart,
)
from .video_abuse_report_reason import VideoReportReasonSchema
from .youtube_list_response import YouTubeListResponse
from .youtube_request import Filter, OptionalParameters, Part, YouTubeRequest
from .youtube_response import YouTubeResponse



from .channel import YouTubeChannel
from .comment_thread import YouTubeCommentThread
from .playlist import YouTubePlaylist
from .playlist_item import YouTubePlaylistItem
from .search import YouTubeSearch
from .video import YouTubeVideo



from enum import Enum
from typing import Optional

from pydantic import BaseModel, Field

from .youtube_request import Filter, OptionalParameters, Part


class SearchPart(Part):
    part: list[str] = Field(default=['snippet'])


class SearchFilter(Filter):
    forContentOwner: Optional[str] = ''
    forDeveloper: Optional[str] = ''
    forMine: Optional[str] = ''


class ChannelType(Enum):
    any = 'any'
    show = 'show'


class EventType(Enum):
    completed = 'completed'
    live = 'live'
    upcoming = 'upcoming'


class Order(Enum):
    date = 'date'
    rating = 'rating'
    relevance = 'relevance'
    title = 'title'
    videoCount = 'videoCount'
    viewCount = 'viewCount'


class VideoType(Enum):
    any = 'any'
    episode = 'episode'
    movie = 'movie'


class VideoSyndicated(Enum):
    any = 'any'
    true = 'true'


class VideoPaidProductPlacement(Enum):
    any = 'any'
    true = 'true'


class VideoLicense(Enum):
    any = 'any'
    creativeCommon = 'creativeCommon'
    youtube = 'youtube'


class VideoEmbeddable(Enum):
    videoEmbeddableUnspecified = 'videoEmbeddableUnspecified'
    true = 'true'
    any = 'any'


class VideoDuration(Enum):
    any = 'any'
    long = 'long'
    medium = 'medium'
    short = 'short'


class VideoDimension(Enum):
    any = 'any'
    two_d = '2d'
    three_d = '3d'


class VideoDefinition(Enum):
    any = 'any'
    high = 'high'
    standard = 'standard'


class VideoCaption(Enum):
    any = 'any'
    closedCaption = 'closedCaption'
    none = 'none'


class Type(Enum):
    video = 'video'
    channel = 'channel'
    playlist = 'playlist'


class SafeSearch(Enum):
    moderate = 'moderate'
    none = 'none'
    strict = 'strict'


class SearchOptionalParameters(OptionalParameters):
    channelId: Optional[str] = ''
    channelType: Optional[str] = ChannelType.any.value
    eventType: Optional[str] = ''
    location: Optional[str] = ''
    locationRadius: Optional[str] = ''
    maxResults: Optional[int] = 25
    onBehalfOfContentOwner: Optional[str] = ''
    order: Optional[str] = Order.relevance.value
    pageToken: Optional[str] = ''
    publishedAfter: Optional[str] = ''
    publishedBefore: Optional[str] = ''
    regionCode: Optional[str] = ''

    q: Optional[str] = ''
    relevanceLanguage: Optional[str] = ''
    safeSearch: Optional[str] = SafeSearch.none.value
    topicId: Optional[str] = ''
    type: Optional[list[str]] = Field(
        default=[Type.channel.value, Type.video.value, Type.playlist.value]
    )

    videoCaption: Optional[str] = ''
    videoCategoryId: Optional[str] = ''
    videoDefinition: Optional[str] = ''
    videoDimension: Optional[str] = ''
    videoDuration: Optional[str] = ''

    videoEmbeddable: Optional[str] = ''
    videoLicense: Optional[str] = VideoLicense.any.value
    videoPaidProductPlacement: Optional[str] = ''
    videoSyndicated: Optional[str] = ''
    videoType: Optional[str] = ''



from collections import defaultdict
from typing import Any

from googleapiclient.http import MediaFileUpload

from ...models import (
    BaseContentDetails,
    BaseSnippet,
    Language,
    LanguageSnippet,
    Region,
    RegionSnippet,
    Video,
    VideoAbuseReportReason,
    VideoAbuseReportReasonSnippet,
    VideoCategory,
    VideoCategorySnippet,
)
from ...schemas import (
    UploadVideo,
    VideoFilter,
    VideoOptionalParameters,
    VideoPart,
    VideoReportReasonSchema,
    YouTubeListResponse,
    YouTubeRatingResponse,
    YouTubeRequest,
)
from ..resource import YouTubeResource


class YouTubeVideo(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)

    def parse_snippet(self, snippet_data: dict[str, Any]) -> dict[str, Any]:
        base_snippet: BaseSnippet = self.parse_base_snippet(snippet_data)
        parsed_snippet: dict[str, Any] = base_snippet.model_dump()
        parsed_snippet['channel_id'] = snippet_data['channelId']
        parsed_snippet['channel_title'] = snippet_data['channelTitle']
        parsed_snippet['live_broadcast_content'] = snippet_data['liveBroadcastContent']
        if snippet_data.get('tags'):
            parsed_snippet['tags'] = snippet_data['tags']
        parsed_snippet['category_id'] = snippet_data['categoryId']
        if snippet_data.get('defaultLanguage'):
            parsed_snippet['default_language'] = snippet_data['defaultLanguage']
        parsed_snippet['localized'] = self.parse_localizations(
            snippet_data['localized']
        )
        if snippet_data.get('defaultAudioLanguage'):
            parsed_snippet['default_audio_language'] = snippet_data[
                'defaultAudioLanguage'
            ]
        return dict(snippet=parsed_snippet)

    def parse_statistics(self, statistics: dict) -> dict:
        parsed_statistics: dict[str, int] = dict()
        parsed_statistics['views_count'] = statistics['viewCount']
        parsed_statistics['likes_count'] = statistics['likeCount']
        parsed_statistics['comments_count'] = statistics['commentCount']
        return dict(statistics=parsed_statistics)

    def parse_region_restriction(self, region_restriction: dict) -> dict:
        parsed_region_restriction: dict[str, list[str]] = dict()
        if region_restriction.get('allowed'):
            parsed_region_restriction['allowed'] = region_restriction.get('allowed')
        else:
            parsed_region_restriction['allowed'] = []
        if region_restriction.get('blocked'):
            parsed_region_restriction['blocked'] = region_restriction.get('blocked')
        else:
            parsed_region_restriction['blocked'] = []
        return dict(region_restriction=parsed_region_restriction)

    def parse_content_details(self, content_details: dict) -> dict:
        base_content_details: BaseContentDetails = self.parse_base_content_details(
            content_details
        )
        parsed_content_details: dict = base_content_details.model_dump()
        parsed_content_details['duration'] = content_details['duration']
        parsed_content_details['dimension'] = content_details['dimension']
        parsed_content_details['definition'] = content_details['definition']
        parsed_content_details['caption'] = content_details['caption']
        parsed_content_details['licensed_content'] = content_details['licensedContent']
        if content_details.get('regionRestriction'):
            parsed_content_details[
                'region_restriction'
            ] = self.parse_region_restriction(content_details['regionRestriction'])
        parsed_content_details['projection'] = content_details['projection']
        if content_details.get('hasCustomThumbnail'):
            parsed_content_details['has_custom_thumbnail'] = content_details[
                'hasCustomThumbnail'
            ]
        return dict(content_details=parsed_content_details)

    def parse_youtube_rating(self, rating_data: dict) -> dict:
        raise NotImplementedError()

    def parse_file_details(self, content_details: dict) -> dict:
        raise NotImplementedError()

    def parse_live_streaming_details(self, content_details: dict) -> dict:
        raise NotImplementedError()

    def parse_player(self, content_details: dict) -> dict:
        raise NotImplementedError()

    def parse_processing_details(self, content_details: dict) -> dict:
        raise NotImplementedError()

    def parse_recording_details(self, content_details: dict) -> dict:
        raise NotImplementedError()

    def parse_status(self, status: dict) -> dict:
        raise NotImplementedError()

    def parse_suggestions(self, suggestions: dict) -> dict:
        raise NotImplementedError()

    def parse_topic_details(self, topic_details: dict) -> dict:
        raise NotImplementedError()

    def parse_user_video_rating(self, rating_response: dict) -> YouTubeListResponse:
        print(rating_response)
        video_ratings: list[dict[str, str]] = [
            {'video_id': rating['videoId'], 'rating': rating['rating']}
            for rating in rating_response['items']
        ]
        rating_response: YouTubeListResponse = YouTubeListResponse(
            etag=rating_response['etag'],
            kind=rating_response['kind'],
            items=video_ratings,
        )
        return rating_response

    def parse_video_categories(
        self, video_categorie_response: dict[str, Any]
    ) -> YouTubeListResponse:
        categories: list[VideoCategory] = [
            VideoCategory(
                id=category['id'],
                snippet=VideoCategorySnippet(
                    title=category['snippet']['title'],
                    assignable=category['snippet']['assignable'],
                    channel_id=category['snippet']['channelId'],
                ),
            )
            for category in video_categorie_response['items']
        ]
        response: YouTubeListResponse = YouTubeListResponse(
            kind=video_categorie_response['kind'],
            etag=video_categorie_response['etag'],
            items=categories,
        )
        return response

    def parse_video_abuse_report_reasons(
        self, video_abuse_report_reasons: dict
    ) -> YouTubeListResponse:
        abuse_reasons: list[VideoAbuseReportReasonSnippet] = [
            VideoAbuseReportReason(
                id=reason['id'],
                snippet=VideoAbuseReportReasonSnippet(label=reason['snippet']['label']),
            )
            for reason in video_abuse_report_reasons['items']
        ]
        response: YouTubeListResponse = YouTubeListResponse(
            kind=video_abuse_report_reasons['kind'],
            etag=video_abuse_report_reasons['etag'],
            items=abuse_reasons,
        )
        return response

    def parse_languages_response(self, languages_response: dict) -> YouTubeListResponse:
        languages: list[Language] = [
            Language(
                id=language['id'],
                snippet=LanguageSnippet(
                    hl=language['snippet']['hl'], name=language['snippet']['name']
                ),
            )
            for language in languages_response['items']
        ]
        response: YouTubeListResponse = YouTubeListResponse(
            kind=languages_response['kind'],
            etag=languages_response['etag'],
            items=languages,
        )
        return response

    def parse_regions_response(self, regions_response: dict) -> YouTubeListResponse:
        regions: list[Region] = [
            Region(
                id=region['id'],
                snippet=RegionSnippet(
                    gl=region['snippet']['gl'], name=region['snippet']['name']
                ),
            )
            for region in regions_response['items']
        ]
        response: YouTubeListResponse = YouTubeListResponse(
            kind=regions_response['kind'],
            etag=regions_response['etag'],
            items=regions,
        )
        return response

    def parse_item(self, item: dict) -> Video:
        id_data: dict = self.parse_id(item)
        snippet_data: dict = self.parse_snippet(item['snippet'])
        statistics_data: dict = self.parse_statistics(item['statistics'])
        content_details: dict = self.parse_content_details(item['contentDetails'])
        id_data.update(snippet_data)
        id_data.update(statistics_data)
        id_data.update(content_details)
        return Video(**id_data)

    def create_upload_request_dict(self, request: UploadVideo) -> dict:
        request_dict: dict = dict()
        if request.snippet:
            request_dict['snippet'] = dict()
            for key, value in request.snippet.model_dump().items():
                if value:
                    request_dict['snippet'][key] = value
        if request.localizations:
            request_dict['localizations'] = dict()
            for key, value in request.localizations.model_dump().items():
                if value:
                    request_dict['localizations'][key] = value
        if request.status:
            request_dict['status'] = dict()
            for key, value in request.status.model_dump().items():
                if value:
                    request_dict['status'][key] = value
        if request.recordingDetails:
            request_dict['recordingDetails'] = dict()
            for key, value in request.recordingDetails.model_dump().items():
                if value:
                    request_dict['recordingDetails'][key] = value
        part: str = ''
        body: dict = defaultdict(dict)
        if request_dict.get('snippet'):
            part += 'snippet'
            body['snippet'] = request_dict['snippet']
        if request_dict.get('localizations'):
            part += ',localizations'
            body['localizations'] = request_dict['localizations']
        if request_dict.get('status'):
            part += ',status'
            body['status'] = request_dict['status']
        if request_dict.get('contentDetails'):
            part += ',contentDetails'
            body['contentDetails'] = request_dict['contentDetails']
        return part, request_dict

    def find_video_by_id(self, video_id: str) -> YouTubeListResponse:
        request_filter: VideoFilter = VideoFilter(id=[video_id])
        part: VideoPart = VideoPart()
        optional_params: VideoOptionalParameters = VideoOptionalParameters()
        request_schema: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(request_schema)
        find_video_request: dict = self.youtube_client.videos().list(**request_dict)
        find_video_result: dict[str, Any] = find_video_request.execute()
        return self.parse_youtube_list_response(find_video_result)

    def find_videos_by_ids(self, video_ids: list[str]) -> YouTubeListResponse:
        filter: VideoFilter = VideoFilter(id=video_ids)
        part: VideoPart = VideoPart()
        optional_params: VideoOptionalParameters = VideoOptionalParameters()
        request_schema: YouTubeRequest = YouTubeRequest(
            part=part, filter=filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(request_schema)
        find_video_request: dict = self.youtube_client.videos().list(**request_dict)
        find_video_result: dict[str, int | str] = find_video_request.execute()
        return self.parse_youtube_list_response(find_video_result)

    def find_most_popular_video_by_region(
        self, region_code: str = 'US', category_id: str = ''
    ) -> list[Video]:
        request_filter: VideoFilter = VideoFilter(chart='mostPopular')
        part: VideoPart = VideoPart()
        optional_params: VideoOptionalParameters = VideoOptionalParameters(
            regionCode=region_code, videoCategoryId=category_id
        )
        request_schema: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(request_schema)
        find_video_request: dict = self.youtube_client.videos().list(**request_dict)
        find_video_result: dict[str, Any] = find_video_request.execute()
        return self.parse_youtube_list_response(find_video_result)

    def get_video_ratings(self, video_ids: list[str]) -> YouTubeListResponse:
        video_ids: str = ','.join(video_ids)
        find_video_rating_request: dict = self.youtube_client.videos().getRating(
            id=video_ids
        )
        find_video_rating_result: dict[
            str, int | str
        ] = find_video_rating_request.execute()
        return self.parse_user_video_rating(find_video_rating_result)

    def rate_video(self, video_id: str, rating: str) -> None:
        """Add a like or dislike rating to a video or remove a rating from a video."""
        rating_request = self.youtube_client.videos().rate(id=video_id, rating=rating)
        rating_request.execute()

    def get_video_categories(self, region_code: str = 'us') -> YouTubeListResponse:
        categories_request = self.youtube_client.videoCategories().list(
            part='snippet', regionCode=region_code
        )
        categories_response = categories_request.execute()
        return self.parse_video_categories(categories_response)

    def upload_video(self, upload_req: UploadVideo) -> Video:
        part, body = self.create_upload_request_dict(upload_req)
        request = self.youtube_client.videos().insert(
            part=part, body=body, media_body=MediaFileUpload(upload_req.file)
        )
        response = request.execute()
        return response

    def update_video(self, upload_req: UploadVideo) -> Video:
        """Updates a video's metadata."""
        part, body = self.create_upload_request_dict(upload_req)
        video_update_request = self.youtube_client.videos().update(
            part=part, body=body, media_body=MediaFileUpload(upload_req.file)
        )
        video_update_response = video_update_request.execute()
        return video_update_response

    def delete_video(self, video_id: str) -> None:
        """Deletes a YouTube video."""
        video_delete_request = self.youtube_client.videos().delete(id=video_id)
        video_delete_request.execute()

    def set_video_thumbnail(self, video_id: str, thumbnail: str) -> YouTubeListResponse:
        set_thumbnail_request = self.youtube_client.thumbnails().set(
            videoId=video_id, media_body=MediaFileUpload(thumbnail)
        )
        set_thumbnail_response = set_thumbnail_request.execute()
        return set_thumbnail_response

    def list_video_abuse_report_reasons(self) -> YouTubeListResponse:
        video_abuse_request = self.youtube_client.videoAbuseReportReasons().list(
            part='snippet'
        )
        video_abuse_response = video_abuse_request.execute()
        return self.parse_video_abuse_report_reasons(video_abuse_response)

    def list_languages(self, language: str = 'en_US') -> YouTubeListResponse:
        list_language_request = self.youtube_client.i18nLanguages().list(
            part='snippet', hl=language
        )
        list_language_response = list_language_request.execute()
        return self.parse_languages_response(list_language_response)

    def list_regions(self, language: str = 'en_US') -> YouTubeListResponse:
        list_language_request = self.youtube_client.i18nRegions().list(
            part='snippet', hl=language
        )
        list_language_response = list_language_request.execute()
        return self.parse_regions_response(list_language_response)

    def report_video_abuse(self, abuse_report: VideoReportReasonSchema) -> None:
        report_abuse_request = self.youtube_client.videos().reportAbuse(
            body={**abuse_report.model_dump()}
        )
        report_abuse_request.execute()



from typing import Optional

from pydantic import BaseModel

from .activity_type import ActivityType
from .content_details import BaseContentDetails
from .resource import Resource
from .snippet import BaseSnippet


class ActivitySnippet(BaseSnippet):
    channel_id: str
    channel_title: str
    type: str
    group_id: Optional[str] = None


class ActivityContentDetails(BaseModel):
    activity_type: ActivityType


class Activity(Resource):
    snippet: BaseSnippet
    content_details: ActivityContentDetails



from typing import Optional

from pydantic import BaseModel, Field

from .youtube_request import Filter, OptionalParameters, Part


class PlaylistPart(Part):
    part: list[str] = Field(
        default=['contentDetails', 'id', 'localizations', 'player', 'snippet', 'status']
    )


class PlaylistFilter(Filter):
    channelId: Optional[str] = ''
    mine: Optional[bool] = None


class PlaylistOptionalParameters(OptionalParameters):
    h1: Optional[str] = ''
    maxResults: Optional[int] = None
    onBehalfOfContentOwner: Optional[str] = ''
    onBehalfOfContentOwnerChannel: Optional[str] = ''
    pageToken: Optional[str] = ''


class CreatePlaylistSnippet(BaseModel):
    title: Optional[str] = ''
    description: Optional[str] = ''
    defaultLanguage: Optional[str] = ''


class CreateStatus(BaseModel):
    privacyStatus: str


class CreatePlaylist(BaseModel):
    snippet: CreatePlaylistSnippet
    status: Optional[CreateStatus] = None



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class Author(BaseModel):
    display_name: str
    profile_image_url: str
    channel_url: Optional[str] = ''
    channel_id: Optional[str] = ''


class Snippet(BaseModel):
    author: Author
    channel_id: str
    video_id: str
    text_display: str
    text_original: str
    can_rate: bool
    viewer_rating: str
    like_count: int
    # moderation_status: str
    published_at: datetime
    updated_at: datetime
    parent_id: Optional[str] = None


class Comment(BaseModel):
    id: str
    snippet: Snippet



from typing import Optional

from pydantic import BaseModel


class VideoReportAbuse(BaseModel):
    video_id: str
    reason_id: str
    secondary_reason_id: Optional[str] = None
    comments: Optional[str] = None
    language: Optional[str] = None



from pydantic import BaseModel, Field

from ..models.resource import Resource


class YouTubeListResponse(BaseModel):
    kind: str
    etag: str
    items: list[Resource] = Field(default_factory=list)



from datetime import datetime
from enum import Enum, auto
from typing import Optional

from pydantic import BaseModel, Field

from .youtube_request import Filter, OptionalParameters, Part


class VideoPart(Part):
    part: list[str] = Field(
        default=[
            'contentDetails',
            'id',
            'liveStreamingDetails',
            'localizations',
            'player',
            'recordingDetails',
            'snippet',
            'statistics',
            'status',
            'topicDetails',
        ]
    )


class Rating(Enum):
    like = auto()
    dislike = auto()


class VideoFilter(Filter):
    chart: Optional[str] = ''
    id: Optional[list[str]] = Field(default_factory=list)
    myRating: Optional[str] = ''


class VideoOptionalParameters(OptionalParameters):
    h1: Optional[str] = ''
    maxHeight: Optional[int] = None
    maxResults: Optional[int] = None
    maxWidth: Optional[int] = None
    onBehalfOfContentOwner: Optional[str] = ''
    pageToken: Optional[str] = ''
    regionCode: Optional[str] = ''
    videoCategoryId: Optional[str] = ''


class UploadVideoSnippet(BaseModel):
    title: str
    description: str
    categoryId: str
    defaultLanguage: Optional[str] = None
    tags: list[str] = Field(default_factory=list)


class UploadVideoLocalizations(BaseModel):
    title: Optional[str] = None
    description: Optional[str] = None


class UploadVideoStatus(BaseModel):
    embeddable: Optional[bool] = None
    license: Optional[str] = None
    privacyStatus: Optional[str] = None
    publicStatsViewable: Optional[bool] = None
    publishAt: Optional[datetime] = None
    selfDeclaredMadeForKids: Optional[str] = None


class UploadVideoRecordingDetails(BaseModel):
    recordingDate: Optional[datetime] = None


class UploadVideo(BaseModel):
    file: str
    snippet: UploadVideoSnippet
    localizations: Optional[UploadVideoLocalizations] = None
    status: Optional[UploadVideoStatus] = None
    recordingDetails: Optional[UploadVideoRecordingDetails] = None



from typing import Any

from ...models import (
    Channel,
    ChannelContentDetails,
    ChannelSnippet,
    ChannelStatistics,
    ChannelStatus,
)
from ...schemas import (
    ChannelFilter,
    ChannelOptionalParameters,
    ChannelPart,
    YouTubeListResponse,
    YouTubeRequest,
)
from ..resource import YouTubeResource


class YouTubeChannel(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)

    def parse_snippet(self, snippet_data: dict[str, Any]) -> ChannelSnippet:
        snippet: dict = dict(**self.parse_base_snippet(snippet_data).model_dump())
        snippet['custom_url'] = snippet_data['customUrl']
        snippet['country'] = snippet_data['country']
        snippet['localized'] = self.parse_localizations(snippet_data['localized'])
        return ChannelSnippet(**snippet)

    def parse_content_details(
        self, content_details_data: dict
    ) -> ChannelContentDetails:
        content_details: dict = dict(
            **self.parse_base_content_details(content_details_data).model_dump()
        )
        content_details['related_playlists'] = content_details_data['relatedPlaylists']
        return ChannelContentDetails(**content_details)

    def parse_statistics(self, statistics_data: dict) -> ChannelStatistics:
        statistics: dict[str, int | bool] = dict()
        statistics['views_count'] = statistics_data['viewCount']
        statistics['subscribers_count'] = statistics_data['subscriberCount']
        statistics['videos_count'] = statistics_data['videoCount']
        statistics['hidden_subscribers_count'] = statistics_data[
            'hiddenSubscriberCount'
        ]
        return ChannelStatistics(**statistics)

    def parse_status(self, status_data: dict) -> ChannelStatus:
        status: dict = dict(**self.parse_base_status(status_data).model_dump())
        status['is_linked'] = status_data['isLinked']
        status['long_uploads_status'] = status_data['longUploadsStatus']
        status['made_for_kids'] = status_data.get('madeForKids', False)
        return ChannelStatus(**status)

    def parse_item(self, item: dict) -> Channel:
        channel_data: dict = dict()
        channel_data.update(self.parse_id(item))
        channel_data.update(dict(snippet=self.parse_snippet(item['snippet'])))
        channel_data.update(
            dict(content_details=self.parse_content_details(item['contentDetails']))
        )
        channel_data.update(dict(statistics=self.parse_statistics(item['statistics'])))
        channel_data.update(dict(status=self.parse_status(item['status'])))
        channel: Channel = Channel(**channel_data)
        return channel

    def find_channel_by_id(self, channel_id: str) -> YouTubeListResponse:
        request_filter: ChannelFilter = ChannelFilter(id=[channel_id])
        part: ChannelPart = ChannelPart()
        optional_params: ChannelOptionalParameters = ChannelOptionalParameters()
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        find_channel_request: dict = self.youtube_client.channels().list(**request_dict)
        find_channel_result: dict[str, int | str] = find_channel_request.execute()
        return self.parse_youtube_list_response(find_channel_result)

    def find_channels_by_ids(self, channel_ids: list[str]) -> YouTubeListResponse:
        filter: ChannelFilter = ChannelFilter(id=channel_ids)
        part: ChannelPart = ChannelPart()
        optional_params: ChannelOptionalParameters = ChannelOptionalParameters()
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        find_channel_request: dict = self.youtube_client.channels().list(**request_dict)
        find_channel_result: dict[str, int | str] = find_channel_request.execute()
        return self.parse_youtube_list_response(find_channel_result)

    def find_my_channel(self) -> YouTubeListResponse:
        filter: ChannelFilter = ChannelFilter(mine=True)
        part: ChannelPart = ChannelPart()
        optional_params: ChannelOptionalParameters = ChannelOptionalParameters()
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        find_channel_request: dict = self.youtube_client.channels().list(**request_dict)
        find_channel_result: dict[str, int | str] = find_channel_request.execute()
        return self.parse_youtube_list_response(find_channel_result)
